/*
 * aku_meter_ep.c
 *
 * Created: 2015-02-27 11:30:59
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "wdog.h"
#include "defs.h"
#include "lcd.h"
#include "analog.h"
#include "clock.h"
#include "ui.h"
#include "settings.h"
#include "serial.h"


#define MAIN_DELAY			20
#define MAIN_VOLTAGE_AUTOSTART	500	
#define SYSTEM_IDLE			20

void HardInit(void);
void FramePrepare(void);
char PowerOutput(char action);
char DisplayBkl(char action);
char DisplayBkl(char action);

extern struct {
	uint16_t vref;
	uint16_t resist;
	uint16_t vin;
	uint16_t vout;
	uint16_t curr;
} analogval;

extern struct {
	char status;
	char restart_bit;
	uint32_t count;
	uint32_t power;

	uint32_t cap;
	uint32_t capact;
	
	uint32_t energy;
	uint32_t energyact;
	uint16_t energy_bit;
} poweracc;

extern struct {
	char press;
	char level;
	char param;
	char enter;
	char refresh;
} ui;

struct {
	char output;
	char display;
	char flash;
	char store;
	char idle;
	char send;
} system;




//---------------------------------------------------
int main(void)
{
	HardInit();
	LCDInit();
	UiInit();
	SettAllRestore();
	SerialInit();
	AnalogInit();
	sei();
	
	LCDLocate(0,0);
	LCDText("    AVT5500     ");
	LCDLocate(1,0);
	LCDText("   AKU  METER   ");
	
	SerialBufferAdd("\r   AKU METER   \r");
	SerialBufferSend();
	ClockInit();
	WdtSet(7);

	//Calibration
	while (system.idle < (SWITCH_REPEAT * 2))
	{
		if (UiScan() == SW_MENUH) 
		{
			ui.level = 5;
			PowerProcess(POWER_PROCESS_START);
		}
		_delay_ms(MAIN_DELAY);
		WdtReset();
		system.idle++;
	}
	
	
	while (1)
	{
		//-------------------------------------------
		if (UiScan())
		{
			UiNavi(ui.press);
			
			if ((ui.level > 0) && (ui.press == SW_MENU))
			system.store = 1;
			
			ui.press = 0;
			ui.refresh = 1;
			system.idle = SYSTEM_IDLE;
			system.flash = 0;
		}
		if ((ui.level == 0) && (system.store))
		{
			SettAllStore();
			system.store = 0;
		}
		

		//-------------------------------------------
		AnalogProc();
		
		//-------------------------------------------
		if (ClockTickCheck())
		{
			AnalogValuesCalc();
			AnalogProcRestart();
			
			//------------------------------------------------
			if (PowerProcess(0) == POWER_PROCESS_IDLE)
			{
				if (analogval.vin >= MAIN_VOLTAGE_AUTOSTART)
				{
					PowerProcess(POWER_PROCESS_START);
					system.send = 1;
				}
			}
			
			//------------------------------------------------
			if (PowerProcess(0) == POWER_PROCESS_WORK)
			{
				PowerCalc();
				PowerOutput(1);
			}
			
			//------------------------------------------------
			FramePrepare();
			
			//------------------------------------------------
			if (PowerAutoStop(0) >= AUTOSTOP_ACCEPT_CYCLE)
			{
				if (PowerProcess(0) == POWER_PROCESS_WORK)
				{
					PowerProcess(POWER_PROCESS_STOP);
					PowerAutoStop(1);
					system.flash = 1;
					system.send = 1;
				}
			}

			//------------------------------------------------
			if (PowerProcess(0) ==  POWER_PROCESS_WORK)
			{
				if ((poweracc.count % SettUsbperiod(0)) == 0) system.send = 1;
			}

			if (system.send)
			{
				system.send = 0;
				SerialBufferSend();
			}

			//------------------------------------------------
			if (PowerProcess(0) != POWER_PROCESS_WORK)
			{
				PowerOutput(2);
			}
			
			if (system.flash)
			{
				if (system.display == 0)
				DisplayBkl(1);
				else
				DisplayBkl(2);
			}
			else
			DisplayBkl(1);
			
			//------------------------------------------------
			if ((system.idle) && (ui.level < 5)) 
			{
				system.idle--;
				if (system.idle == 0)
				{
					ui.level = 0;
					ui.enter = 0;
				}
			}
			ui.refresh = 1;
		}
		
		//-------------------------------------------
		if (ui.refresh)
		{
			UiMenu(ui.level);
			ui.refresh = 0;
			
		}
		
		//-------------------------------------------
		_delay_ms(MAIN_DELAY);
		WdtReset();
	}
}


//---------------------------------------------------
void HardInit(void)
{
	ANALOG_PORT = 0x00;
	ANALOG_DDR = 0x00;
	
	UI_PORT = (1<<SW1_PIN | 1<<SW2_PIN | 1<<SW3_PIN);
	UI_DDR = (1<<BKL_PIN | 1<<REL_PIN);
	
	LCD_PORT = 0xFF;
	LCD_DDR = 0x3F;
	
	ACSR = (1<<ACD);
}

//---------------------------------------------------
void FramePrepare(void)
{
	SerialBufferClear();
	SerialBufferAdd("\r*");	
	SerialBufferAdd(UiTime2str(poweracc.count));
	SerialBufferAdd("; ");
	
	if (poweracc.capact >= 1000)
		SerialBufferAdd(Ui4Val2str((poweracc.capact / 10), 3));
	else
		SerialBufferAdd(Ui4Val2str(poweracc.capact, 4));	
	SerialBufferAdd("Ah; ");
	
	if (poweracc.energyact >= 1000)
		SerialBufferAdd(Ui4Val2str((poweracc.energyact / 100), 2));
	else
		SerialBufferAdd(Ui4Val2str(poweracc.energyact, 4));
	SerialBufferAdd("Wh; ");
	
	if (poweracc.power >= 1000)
		SerialBufferAdd(Ui4Val2str((poweracc.power / 100), 2));
	else
		SerialBufferAdd(Ui4Val2str(poweracc.power, 4));
	SerialBufferAdd("W ; ");
	
	if (SettVchannel(0) == 0)
	{
		SerialBufferAdd(Ui4Val2str((analogval.vin / 10), 3));
		SerialBufferAdd("Vi; ");	
	}
	else
	{
		SerialBufferAdd(Ui4Val2str((analogval.vout / 10), 3));
		SerialBufferAdd("Vo; ");
	}
	
	SerialBufferAdd(Ui4Val2str(analogval.curr / 10, 3));
	SerialBufferAdd("A ;");
	//SerialBufferSend();	
}


//----------------------------------
char PowerOutput(char action)
{
	if (action == 1)
	{
		UI_PORT |= (1<<REL_PIN);
		system.output = 1;
	}
		
	if (action == 2)
	{
		UI_PORT &= ~(1<<REL_PIN);
		system.output = 0;
	}
	
	return system.output;
}

//----------------------------------
char DisplayBkl(char action)
{
	if (action == 1)
	{
		UI_PORT |= (1<<BKL_PIN);
		system.display = 1;
	}
	
	if (action == 2)
	{
		UI_PORT &= ~(1<<BKL_PIN);
		system.display = 0;
	}
	
	return system.display;
}